/**************************************************************************//**
 * @file     os_tick_timer.c
 * @brief    CMSIS OS Tick implementation for OS Timer
 * @version  V1.0.0
 * @date     05. July 2017
 ******************************************************************************/
/*
 * Copyright (c) 2017-2017 ARM Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "os_tick.h"
#include "irq_ctrl.h"

#include "RTE_Components.h"
#include CMSIS_device_header

// Define Periodic Interval Timer interrupt priority and id number
#ifndef PIT_IRQ_PRIORITY
#define PIT_IRQ_PRIORITY           7U
#endif
#ifndef PIT_ID
#define PIT_ID                     3U
#endif

// Periodic Interval Timer tick frequency
static uint32_t PIT_Clock;

// Periodic Interval Timer interrupt pending flag
static uint8_t  PIT_PendIRQ;

// Setup OS Tick
int32_t OS_Tick_Setup (uint32_t freq, IRQHandler_t handler) {

  if (freq == 0U) {
    return (-1);
  }

  // Calculate timer clock
  PIT_Clock = SystemCoreClock / (((PMC->PMC_MCKR & PMC_MCKR_H32MXDIV_H32MXDIV2) ? (2U) : (1U)) * 16U);

  // Calculate and set load value
  PIT->PIT_MR = ((PIT_Clock / freq) - 1U) & PIT_MR_PIV_Msk;

  PIT_PendIRQ = 0U;

  // Disable corresponding IRQ
  IRQ_Disable     (PIT_ID);
  IRQ_ClearPending(PIT_ID);

  // Set lowest interrupt priority
  IRQ_SetPriority (PIT_ID, PIT_IRQ_PRIORITY);

  // Set edge-triggered IRQ
  IRQ_SetMode (PIT_ID, IRQ_MODE_TRIG_EDGE);

  // Register tick interrupt handler function
  IRQ_SetHandler(PIT_ID, handler);

  // Enable corresponding IRQ
  PIT->PIT_MR |= PIT_MR_PITIEN;
  IRQ_Enable (PIT_ID);

  return (0);
}

// Enable OS Tick
__WEAK int32_t  OS_Tick_Enable (void) {

  if (PIT_PendIRQ != 0U) {
    PIT_PendIRQ = 0U;
    IRQ_SetPending (PIT_ID);
  }

  // Start the counter
  PIT->PIT_MR |= PIT_MR_PITEN;

  return (0);
}

// Disable OS Tick
__WEAK int32_t  OS_Tick_Disable (void) {

  // Stop the counter
  PIT->PIT_MR &= ~PIT_MR_PITEN;

  if (IRQ_GetPending(PIT_ID) != 0) {
    IRQ_ClearPending (PIT_ID);
    PIT_PendIRQ = 1U;
  }

  return (0);
}

// Acknowledge OS Tick IRQ
int32_t OS_Tick_AcknowledgeIRQ (void) {
  uint32_t status = PIT->PIT_SR;
  uint32_t pivr = PIT->PIT_PIVR;
  return (0);
}

// Get OS Tick IRQ number
int32_t  OS_Tick_GetIRQn (void) {
  return (PIT_ID);
}

// Get OS Tick clock
uint32_t OS_Tick_GetClock (void) {
  return PIT_Clock;
}

// Get OS Tick interval
uint32_t OS_Tick_GetInterval (void) {
  return PIT->PIT_MR & PIT_MR_PIV_Msk;
}

// Get OS Tick count value
uint32_t OS_Tick_GetCount (void) {
  return PIT->PIT_PIIR & PIT_PIIR_CPIV_Msk;
}

// Get OS Tick overflow status
uint32_t OS_Tick_GetOverflow (void) {
  return ((PIT->PIT_PIIR & PIT_PIIR_PICNT_Msk) ? (1U) : (0U));
}
